@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -19,10 +19,10 @@ our $VERSION = '1.699_013';
 
 use Carp ();
 
-my $_has_threads;
+my ($_has_threads, $_freeze, $_thaw, $_tid, $_oid);
 
 BEGIN {
-   local $@; local $SIG{__DIE__} = \&_NOOP;
+   local $@; local $SIG{__DIE__};
 
    ## Forking is emulated under the Windows enviornment, excluding Cygwin.
    ## MCE 1.514+ will load the 'threads' module by default on Windows.
@@ -37,24 +37,41 @@ BEGIN {
       }
    }
 
-   $_has_threads = $INC{'threads/shared.pm'} ? 1 : 0;
+   $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+   $_tid = $_has_threads ? threads->tid() : 0;
+   $_oid = "$$.$_tid";
 
    eval 'PDL::no_clone_skip_warning()' if $INC{'PDL.pm'};
+   eval 'use PDL::IO::Storable' if $INC{'PDL.pm'};
+
+   if (!exists $INC{'PDL.pm'}) {
+      eval 'use Sereal qw( encode_sereal decode_sereal )';
+      if ( !$@ ) {
+         $_freeze = sub { encode_sereal( @_, { freeze_callbacks => 1 } ) };
+         $_thaw   = \&decode_sereal;
+      }
+   }
+   if (!defined $_freeze) {
+      require Storable;
+      $_freeze = \&Storable::freeze;
+      $_thaw   = \&Storable::thaw;
+   }
+
+   return;
 }
 
-use Scalar::Util qw( looks_like_number refaddr );
+use Scalar::Util qw( looks_like_number refaddr weaken );
 use Time::HiRes qw( sleep time );
 
 use Symbol qw( qualify_to_ref );
 use Socket qw( SOL_SOCKET SO_RCVBUF );
-use Storable ();
 
 use MCE::Util qw( $LF );
 use MCE::Signal;
 use MCE::Mutex;
 use bytes;
 
-our ($MCE, $_que_template, $_que_read_size);
+our ($MCE, $RLA, $_que_template, $_que_read_size);
 our (%_valid_fields_new);
 
 my  ($TOP_HDLR, $_is_MSWin32, $_is_winenv, $_prev_mce);
@@ -75,11 +92,11 @@ BEGIN {
    ## _chunk_id _mce_sid _mce_tid _pids _run_mode _single_dim _thrs _tids _wid
    ## _exiting _exit_pid _total_exited _total_running _total_workers _task_wid
    ## _send_cnt _sess_dir _spawned _state _status _task _task_id _wrk_status
-   ## _init_total_workers _last_sref _mgr_live _rla_data _rla_return
+   ## _init_pid _init_total_workers _last_sref _mgr_live _rla_data _rla_return
    ##
    ## _bsb_r_sock _bsb_w_sock _bse_r_sock _bse_w_sock _com_r_sock _com_w_sock
    ## _dat_r_sock _dat_w_sock _que_r_sock _que_w_sock _rla_r_sock _rla_w_sock
-   ## _data_channels _lock_chn _mutex_n
+   ## _data_channels _lock_chn _mutex_n _caller
 
    %_valid_fields_new = map { $_ => 1 } qw(
       max_workers tmp_dir use_threads user_tasks task_end task_name freeze thaw
@@ -112,23 +129,25 @@ BEGIN {
       chunk_size max_retries max_workers task_name tmp_dir user_args
    )) {
       *{ $_p } = sub () {
-         my $x = shift; my $self = ref($x) ? $x : $MCE;
+         my $self = shift; $self = $MCE unless ref($self);
          return $self->{$_p};
       };
    }
    for my $_p (qw( chunk_id sess_dir task_id task_wid wid )) {
       *{ $_p } = sub () {
-         my $x = shift; my $self = ref($x) ? $x : $MCE;
+         my $self = shift; $self = $MCE unless ref($self);
          return $self->{"_${_p}"};
       };
    }
    for my $_p (qw( freeze thaw )) {
       *{ $_p } = sub () {
-         my $x = shift; my $self = ref($x) ? $x : $MCE;
+         my $self = shift; $self = $MCE unless ref($self);
          return $self->{$_p}(@_);
       };
    }
 
+   $RLA = {};
+
    return;
 }
 
@@ -141,67 +160,49 @@ BEGIN {
 use constant { SELF => 0, CHUNK => 1, CID => 2 };
 
 our $_MCE_LOCK : shared = 1;
-our $_RUN_LOCK : shared = 1;
-our $_WIN_LOCK : shared = 1;
-our $_EXT_LOCK : shared = 1;
 
-my  $TMP_DIR = $MCE::Signal::tmp_dir;
-my  $FREEZE  = \&Storable::freeze;
-my  $THAW    = \&Storable::thaw;
-
-my ($MAX_WORKERS, $CHUNK_SIZE) = (1, 1);
-my ($_imported);
+my ($_def, $_imported) = ({});
 
 sub import {
-   my $_class = shift; return if ($_imported++);
+
+   my ($_class, $_pkg) = (shift, caller);
+   my $_p = $_def->{$_pkg} = {};
 
    ## Process module arguments.
    while ( my $_argument = shift ) {
       my $_arg = lc $_argument;
 
-      $MAX_WORKERS = shift, next if ( $_arg eq 'max_workers' );
-      $CHUNK_SIZE  = shift, next if ( $_arg eq 'chunk_size' );
-      $FREEZE      = shift, next if ( $_arg eq 'freeze' );
-      $THAW        = shift, next if ( $_arg eq 'thaw' );
+      $_p->{MAX_WORKERS} = shift, next if ( $_arg eq 'max_workers' );
+      $_p->{CHUNK_SIZE}  = shift, next if ( $_arg eq 'chunk_size' );
+      $_p->{TMP_DIR}     = shift, next if ( $_arg eq 'tmp_dir' );
+      $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
+      $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
 
-      if ( $_arg eq 'sereal' ) {
-         if (shift eq '1') {
-            local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            $FREEZE = \&encode_sereal, $THAW = \&decode_sereal unless $@;
-         }
-         next;
-      }
-      if ( $_arg eq 'tmp_dir' ) {
-         $TMP_DIR = shift;
-         my $_e1 = 'is not a directory or does not exist';
-         my $_e2 = 'is not writeable';
-         _croak("Error: ($TMP_DIR) $_e1") unless -d $TMP_DIR;
-         _croak("Error: ($TMP_DIR) $_e2") unless -w $TMP_DIR;
-         next;
-      }
       if ( $_arg eq 'export_const' || $_arg eq 'const' ) {
-         if (shift eq '1') {
+         if ( shift eq '1' ) {
             no strict 'refs'; no warnings 'redefine';
-            my $_package = caller;
-            *{ $_package . '::SELF'  } = \&SELF;
-            *{ $_package . '::CHUNK' } = \&CHUNK;
-            *{ $_package . '::CID'   } = \&CID;
+            *{ $_pkg.'::SELF'  } = \&SELF;
+            *{ $_pkg.'::CHUNK' } = \&CHUNK;
+            *{ $_pkg.'::CID'   } = \&CID;
          }
          next;
       }
 
+      ## Sereal, if available, is used automatically by MCE 1.800 onwards.
+      next if ( $_arg eq 'sereal' );
+
       _croak("Error: ($_argument) invalid module option");
    }
 
+   return if $_imported++;
+
    ## Preload essential modules.
    require MCE::Core::Validation;
    require MCE::Core::Manager;
    require MCE::Core::Worker;
 
-   {
-      no strict 'refs'; no warnings 'redefine';
-      *{ 'MCE::_parse_max_workers' } = \&MCE::Util::_parse_max_workers;
-   }
+   no strict 'refs'; no warnings 'redefine';
+   *{ 'MCE::_parse_max_workers' } = \&MCE::Util::_parse_max_workers;
 
    ## Instantiate a module-level instance.
    $MCE = MCE->new( _module_instance => 1, max_workers => 0 );
@@ -217,10 +218,11 @@ sub import {
 
 use constant {
 
-   FAST_SEND_SIZE => 1024 * 64 + 128,   # Use one print call if N < size
    MAX_CHUNK_SIZE => 1024 * 1024 * 64,  # Maximum chunk size allowed
 
-   DATA_CHANNELS  => 8,        # Max data channels
+   # Max data channels. This cannot be greater than 8 on MSWin32.
+   DATA_CHANNELS  => ($^O eq 'MSWin32') ? 8 : 12,
+
    MAX_RECS_SIZE  => 8192,     # Reads number of records if N <= value
                                # Reads number of bytes if N > value
 
@@ -261,27 +263,19 @@ use constant {
    WANTS_REF      => 3         # Callee wants H/A/S ref
 };
 
-my (%_mce_sess_dir, %_mce_spawned); my $_mce_count = 0;
+my $_mce_count = 0;
 
-MCE::Signal::_set_session_vars(\%_mce_sess_dir, \%_mce_spawned);
-
-sub _clean_sessions {
-   my ($_mce_sid) = @_;
-   for my $_s (keys %_mce_spawned) {
-      delete $_mce_spawned{$_s} unless ($_s eq $_mce_sid);
-   }
-   return;
-}
-sub _clear_session {
-   my ($_mce_sid) = @_;
-   delete $_mce_spawned{$_mce_sid};
-   for my $_s (keys %_mce_spawned) {
-      (delete $_mce_spawned{$_s})->shutdown(1);
-   }
-   return;
+sub CLONE {
+   $_tid = threads->tid() if $_has_threads;
 }
 
-sub DESTROY {}
+sub DESTROY {
+   $_[0]->shutdown(1) if ( $_[0] && $_[0]->{_init_pid} eq "$$.$_tid" );
+}
+
+END {
+   $MCE = $TOP_HDLR = undef;
+}
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -339,24 +333,24 @@ sub _save_state    { $_prev_mce = $MCE; return; }
 sub new {
 
    my ($class, %self) = @_;
+   my $_pkg = exists $self{pkg} ? delete $self{pkg} : caller;
 
    @_ = ();
 
    bless(\%self, ref($class) || $class);
 
-   ## Public options.
-   $self{max_workers}  ||= $MAX_WORKERS;
-   $self{chunk_size}   ||= $CHUNK_SIZE;
-   $self{tmp_dir}      ||= $TMP_DIR;
-   $self{freeze}       ||= $FREEZE;
-   $self{thaw}         ||= $THAW;
-   $self{task_name}    ||= 'MCE';
+   $self{task_name}   ||= 'MCE';
+   $self{max_workers} ||= $_def->{$_pkg}{MAX_WORKERS} || 1;
+   $self{chunk_size}  ||= $_def->{$_pkg}{CHUNK_SIZE}  || 1;
+   $self{tmp_dir}     ||= $_def->{$_pkg}{TMP_DIR}     || $MCE::Signal::tmp_dir;
+   $self{freeze}      ||= $_def->{$_pkg}{FREEZE}      || $_freeze;
+   $self{thaw}        ||= $_def->{$_pkg}{THAW}        || $_thaw;
 
    if (exists $self{_module_instance}) {
       $self{_init_total_workers} = $self{max_workers};
       $self{_chunk_id} = $self{_task_wid} = $self{_wrk_status} = 0;
       $self{_spawned}  = $self{_task_id}  = $self{_wid} = 0;
-      $self{_data_channels} = 1;
+      $self{_init_pid} = 0;
 
       return \%self;
    }
@@ -366,6 +360,8 @@ sub new {
          unless (exists $_valid_fields_new{$_p});
    }
 
+   $self{_caller} = $_pkg, $self{_init_pid} = "$$.$_tid";
+
    if (defined $self{use_threads}) {
       if (!$_has_threads && $self{use_threads} ne '0') {
          my $_msg  = "\n";
@@ -431,12 +427,12 @@ sub new {
    $self{_wid}        = 0;  # Worker ID, starts at 1 per MCE instance
    $self{_wrk_status} = 0;  # For saving exit status when worker exits
 
-   $self{chunk_size} = MAX_CHUNK_SIZE if ($self{chunk_size} > MAX_CHUNK_SIZE);
+   $self{chunk_size}  = MAX_CHUNK_SIZE if ($self{chunk_size} > MAX_CHUNK_SIZE);
 
-   $self{_last_sref} = (ref $self{input_data} eq 'SCALAR')
+   $self{_last_sref}  = (ref $self{input_data} eq 'SCALAR')
       ? refaddr($self{input_data}) : 0;
 
-   my $_data_channels = DATA_CHANNELS;
+   my $_data_channels = ($_oid eq "$$.$_tid") ? DATA_CHANNELS : 4;
    my $_total_workers = 0;
 
    if (defined $self{user_tasks}) {
@@ -466,17 +462,17 @@ sub new {
 
 sub spawn {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    @_ = ();
 
    _croak('MCE::spawn: method is not allowed by the worker process')
       if ($self->{_wid});
 
-   ## Return if workers have already been spawned.
-   return $self if ($self->{_spawned});
+   ## Return if workers have already been spawned or if module instance.
+   return $self if ($self->{_spawned} || exists $self->{_module_instance});
 
-   ## The shared server must be running if present.
+   ## Start the shared-manager process if present.
    MCE::Shared::start() if ($INC{'MCE/Shared.pm'});
 
    ## Load input module.
@@ -505,25 +501,17 @@ sub spawn {
       require POSIX unless $INC{'POSIX.pm'};
    }
 
-   lock $_MCE_LOCK if ($_has_threads);  # Obtain MCE lock
-   lock $_WIN_LOCK if ($_is_MSWin32);
+   lock $_MCE_LOCK if ($self->{use_threads});  # Obtain MCE lock
 
    my $_die_handler  = $SIG{__DIE__};  $SIG{__DIE__}  = \&_die;
    my $_warn_handler = $SIG{__WARN__}; $SIG{__WARN__} = \&_warn;
 
-   if (!defined $TOP_HDLR) {
-      $TOP_HDLR = $self;
-   }
-   elsif (!$TOP_HDLR->{_mgr_live} && !$TOP_HDLR->{_wid}) {
-      $TOP_HDLR->shutdown if ($_is_MSWin32);
+   if (!defined $TOP_HDLR || (!$TOP_HDLR->{_mgr_live} && !$TOP_HDLR->{_wid})) {
       $TOP_HDLR = $self;
    }
    elsif (refaddr($self) != refaddr($TOP_HDLR)) {
-      _croak('Running parallel MCE instances is not supported on Windows')
-         if ($_is_MSWin32);
-
-      $self->{_data_channels} = 1 if ($self->{_data_channels} > 1);
-      $self->{_lock_chn} = 1 if ($self->{_init_total_workers} > 1);
+      $self->{_data_channels} = 4 if ($self->{_data_channels} > 4);
+      $self->{_lock_chn}      = 1 if ($self->{_init_total_workers} > 4);
    }
 
    ## Configure tid/sid for this instance here, not in the new method above.
@@ -551,8 +539,6 @@ sub spawn {
 
       $_sess_dir = $self->{_sess_dir} = "$_tmp_dir/$_mce_sid." . (++$_cnt)
          while ( !(mkdir $_sess_dir, 0770) );
-
-      $_mce_sess_dir{$_sess_dir} = 1;
    }
 
    ## -------------------------------------------------------------------------
@@ -593,8 +579,6 @@ sub spawn {
    ## -------------------------------------------------------------------------
 
    ## Spawn workers.
-   $_mce_spawned{$_mce_sid} = $self;
-
    $self->{_pids}   = [], $self->{_thrs}  = [], $self->{_tids} = [];
    $self->{_status} = [], $self->{_state} = [], $self->{_task} = [];
 
@@ -690,7 +674,7 @@ sub forseq {
 
 sub process {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _validate_runstate($self, 'MCE::process');
 
@@ -738,7 +722,7 @@ sub relay (;&) {
 
 sub restart_worker {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    @_ = ();
 
@@ -770,7 +754,7 @@ sub restart_worker {
       _dispatch_child($self, $_wid, $_task, $_task_id, $_task_wid, $_params);
    }
 
-   sleep 0.001;
+   sleep 0.001;  # this sleeps for 0.015 on Windows and Cygwin
 
    return;
 }
@@ -783,7 +767,7 @@ sub restart_worker {
 
 sub run {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::run: method is not allowed by the worker process')
       if ($self->{_wid});
@@ -971,8 +955,7 @@ sub run {
       local $\ = undef; local $/ = $LF;
 
       ## Obtain lock.
-      lock $_MCE_LOCK if ($_has_threads &&  $_is_winenv);
-      lock $_RUN_LOCK if ($_has_threads && !$_is_winenv);
+      lock $_MCE_LOCK if ($self->{use_threads});
 
       my ($_frozen_nodata, $_wid, %_task0_wids);
       my $_BSE_W_SOCK    = $self->{_bse_w_sock};
@@ -994,14 +977,14 @@ sub run {
 
       ## Submit params data to workers.
       for my $_i (1 .. $_total_workers) {
-         print {$_COM_R_SOCK} $_i . $LF;
+         print {$_COM_R_SOCK} $_i.$LF;
          chomp($_wid = <$_COM_R_SOCK>);
 
          if (!$_has_user_tasks || exists $_task0_wids{$_wid}) {
-            print {$_COM_R_SOCK} length($_frozen_params) . $LF . $_frozen_params;
+            print {$_COM_R_SOCK} length($_frozen_params).$LF, $_frozen_params;
             $self->{_state}->[$_wid]->{_params} = \%_params;
          } else {
-            print {$_COM_R_SOCK} length($_frozen_nodata) . $LF . $_frozen_nodata;
+            print {$_COM_R_SOCK} length($_frozen_nodata).$LF, $_frozen_nodata;
             $self->{_state}->[$_wid]->{_params} = \%_params_nodata;
          }
 
@@ -1078,7 +1061,7 @@ sub run {
 
 sub send {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::send: method is not allowed by the worker process')
       if ($self->{_wid});
@@ -1123,17 +1106,10 @@ sub send {
    my $_len          = length $_frozen_data;
 
    ## Submit data to worker.
-   print {$_COM_R_SOCK} '_data' . $LF;
-
+   print {$_COM_R_SOCK} '_data'.$LF;
    <$_COM_R_SOCK>;
 
-   if ($_len < FAST_SEND_SIZE) {
-      print {$_COM_R_SOCK} $_len . $LF . $_frozen_data;
-   } else {
-      print {$_COM_R_SOCK} $_len . $LF;
-      print {$_COM_R_SOCK} $_frozen_data;
-   }
-
+   print {$_COM_R_SOCK} $_len.$LF, $_frozen_data;
    <$_COM_R_SOCK>;
 
    if (defined $_submit_delay && $_submit_delay > 0.0) {
@@ -1153,7 +1129,7 @@ sub send {
 
 sub shutdown {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
    my $_no_lock = shift || 0;
 
    @_ = ();
@@ -1162,15 +1138,15 @@ sub shutdown {
    return unless (defined $MCE::Signal::tmp_dir);
    return unless ($self->{_spawned});
 
-   ## Wait for workers to complete processing before shutting down.
    _validate_runstate($self, 'MCE::shutdown');
 
+   ## Wait for workers to complete processing before shutting down.
    $self->run(0) if ($self->{_send_cnt});
 
    local $SIG{__DIE__}  = \&_die;
    local $SIG{__WARN__} = \&_warn;
 
-   lock $_MCE_LOCK if ($_has_threads && ! $_no_lock);
+   lock $_MCE_LOCK if ($self->{use_threads} && !$_no_lock);
 
    my $_COM_R_SOCK     = $self->{_com_r_sock};
    my $_data_channels  = $self->{_data_channels};
@@ -1178,9 +1154,6 @@ sub shutdown {
    my $_sess_dir       = $self->{_sess_dir};
    my $_mce_sid        = $self->{_mce_sid};
 
-   ## Delete entry.
-   delete $_mce_spawned{$_mce_sid};
-
    if (defined $TOP_HDLR && refaddr($self) == refaddr($TOP_HDLR)) {
       $TOP_HDLR = undef;
    }
@@ -1190,12 +1163,9 @@ sub shutdown {
    ## Notify workers to exit loop.
    local ($!, $?); local $\ = undef; local $/ = $LF;
 
-   {
-      lock $_EXT_LOCK if $_is_MSWin32;
-      for (1 .. $_total_workers) {
-         print {$_COM_R_SOCK} '_exit' . $LF;
-         <$_COM_R_SOCK>;
-      }
+   for (1 .. $_total_workers) {
+      print {$_COM_R_SOCK} '_exit'.$LF;
+      <$_COM_R_SOCK>;
    }
 
    ## Reap children and/or threads.
@@ -1237,12 +1207,11 @@ sub shutdown {
          }
       }
       rmdir "$_sess_dir";
-      delete $_mce_sess_dir{$_sess_dir};
    }
 
    ## Reset instance.
-   @{$self->{_pids}}  = (), @{$self->{_thrs}}   = (), @{$self->{_tids}} = ();
-   @{$self->{_state}} = (), @{$self->{_status}} = (), @{$self->{_task}} = ();
+   $self->{_pids}  = $self->{_thrs}   = $self->{_tids} = undef;
+   $self->{_state} = $self->{_status} = $self->{_task} = undef;
 
    $self->{_mce_sid}  = $self->{_mce_tid}  = $self->{_sess_dir} = undef;
    $self->{_chunk_id} = $self->{_send_cnt} = $self->{_spawned}  = 0;
@@ -1261,7 +1230,7 @@ sub shutdown {
 
 sub sync {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    return unless ($self->{_wid});
 
@@ -1279,15 +1248,17 @@ sub sync {
    local $\ = undef if (defined $\); local $/ = $LF if (!$/ || $/ ne $LF);
 
    ## Notify the manager process (begin).
-   print {$_DAT_W_SOCK} OUTPUT_B_SYN . $LF . $_chn . $LF;
+   print {$_DAT_W_SOCK} OUTPUT_B_SYN.$LF . $_chn.$LF;
 
    ## Wait here until all workers (task_id 0) have synced.
+   MCE::Util::_sock_ready($_BSB_R_SOCK) if $_is_MSWin32;
    1 until sysread $_BSB_R_SOCK, $_buffer, 1;
 
    ## Notify the manager process (end).
-   print {$_DAT_W_SOCK} OUTPUT_E_SYN . $LF . $_chn . $LF;
+   print {$_DAT_W_SOCK} OUTPUT_E_SYN.$LF . $_chn.$LF;
 
    ## Wait here until all workers (task_id 0) have un-synced.
+   MCE::Util::_sock_ready($_BSE_R_SOCK) if $_is_MSWin32;
    1 until sysread $_BSE_R_SOCK, $_buffer, 1;
 
    return;
@@ -1295,7 +1266,7 @@ sub sync {
 
 sub yield {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    return unless ($self->{_i_wrk_st});
    return unless ($self->{_task_wid});
@@ -1310,7 +1281,7 @@ sub yield {
 
    sleep $_delay if ($_delay > 0.0);
 
-   if ($_count && $_count > 2_000_000_000) {
+   if ($_count && $_count > 2e9) {
       $self->{_i_wrk_st} = time;
    }
 
@@ -1327,7 +1298,7 @@ sub yield {
 
 sub abort {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    my $_QUE_R_SOCK = $self->{_que_r_sock};
    my $_QUE_W_SOCK = $self->{_que_w_sock};
@@ -1351,11 +1322,11 @@ sub abort {
          $_DAT_LOCK->lock() if $_lock_chn;
 
          if (exists $self->{_rla_return}) {
-            print {$_DAT_W_SOCK} OUTPUT_W_RLA . $LF . $_chn . $LF;
-            print {$_DAU_W_SOCK} (delete $self->{_rla_return}) . $LF;
+            print {$_DAT_W_SOCK} OUTPUT_W_RLA.$LF . $_chn.$LF;
+            print {$_DAU_W_SOCK} (delete $self->{_rla_return}).$LF;
          }
 
-         print {$_DAT_W_SOCK} OUTPUT_W_ABT . $LF . $_chn . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_W_ABT.$LF . $_chn.$LF;
 
          $_DAT_LOCK->unlock() if $_lock_chn;
       }
@@ -1368,7 +1339,7 @@ sub abort {
 
 sub exit {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    my $_exit_status = (defined $_[0]) ? $_[0] : $?;
    my $_exit_msg    = (defined $_[1]) ? $_[1] : '';
@@ -1379,10 +1350,6 @@ sub exit {
    _croak('MCE::exit: method is not allowed by the manager process')
       unless ($self->{_wid});
 
-   MCE::Signal::stop_and_exit('__DIE__') unless ($self->{_running});
-
-   _clear_session( $self->{_mce_sid} );
-
    my $_chn        = $self->{_chn};
    my $_DAT_LOCK   = $self->{_dat_lock};
    my $_DAT_W_SOCK = $self->{_dat_w_sock}->[0];
@@ -1402,34 +1369,34 @@ sub exit {
       $_DAT_LOCK->lock() if $_lock_chn;
 
       if (exists $self->{_rla_return}) {
-         print {$_DAT_W_SOCK} OUTPUT_W_RLA . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} (delete $self->{_rla_return}) . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_W_RLA.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} (delete $self->{_rla_return}).$LF;
       }
 
-      print {$_DAT_W_SOCK} OUTPUT_W_EXT . $LF . $_chn . $LF;
+      print {$_DAT_W_SOCK} OUTPUT_W_EXT.$LF . $_chn.$LF;
       print {$_DAU_W_SOCK}
-         $_task_id . $LF . $self->{_wid} . $LF . $self->{_exit_pid} . $LF .
-         $_exit_status . $LF . $_exit_id . $LF . $_len . $LF . $_exit_msg
+         $_task_id.$LF . $self->{_wid}.$LF . $self->{_exit_pid}.$LF .
+         $_exit_status.$LF . $_exit_id.$LF . $_len.$LF . $_exit_msg
       ;
 
       if ($self->{_retry} && $self->{_retry}->[2]--) {
          my $_buf = $self->{freeze}($self->{_retry});
-         print {$_DAU_W_SOCK} length($_buf) . $LF . $_buf;
+         print {$_DAU_W_SOCK} length($_buf).$LF, $_buf;
       }
       else {
-         print {$_DAU_W_SOCK} '0' . $LF;
+         print {$_DAU_W_SOCK} '0'.$LF;
       }
 
-      <$_DAU_W_SOCK>;
+      <$_DAU_W_SOCK> unless ($^O eq 'MSWin32');
 
       $_DAT_LOCK->unlock() if $_lock_chn;
    }
 
-   ## Exit thread/child process.
-   $SIG{__DIE__} = $SIG{__WARN__} = sub {};
+   ## Check for any Hobo workers not yet joined.
+   MCE::Hobo->finish() if $INC{'MCE/Hobo.pm'};
 
-   if ($_has_threads && threads->can('exit')) {
-      if ($_is_MSWin32) { lock $_EXT_LOCK; sleep 0.002; }
+   ## Exit thread/child process.
+   if ($self->{use_threads} || ($_has_threads && $_is_MSWin32)) {
       threads->exit($_exit_status);
    }
    elsif ($self->{posix_exit}) {
@@ -1444,7 +1411,7 @@ sub exit {
 
 sub last {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::last: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1458,7 +1425,7 @@ sub last {
 
 sub next {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::next: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1472,7 +1439,7 @@ sub next {
 
 sub pid {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    if (defined $self->{_pid}) {
       $self->{_pid};
@@ -1488,7 +1455,7 @@ sub pid {
 
 sub status {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::status: method is not allowed by the worker process')
       if ($self->{_wid});
@@ -1506,7 +1473,7 @@ sub status {
 
 sub do {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::do: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1528,7 +1495,7 @@ sub do {
 
 sub gather {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    _croak('MCE::gather: method is not allowed by the manager process')
       unless ($self->{_wid});
@@ -1551,7 +1518,7 @@ sub gather {
 
    sub sendto {
 
-      my $x = shift; my $self = ref($x) ? $x : $MCE;
+      my $self = shift; $self = $MCE unless ref($self);
       my $_to = shift;
 
       _croak('MCE::sendto: method is not allowed by the manager process')
@@ -1564,6 +1531,12 @@ sub gather {
 
       if (!defined $_dest) {
          if (ref $_to && defined (my $_fd = fileno($_to))) {
+            if (my $_ob = tied *{ $_to }) {
+               if (ref $_ob eq 'IO::TieCombine::Handle') {
+                  $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
+                  $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
+               }
+            }
             my $_data_ref = (scalar @_ == 1) ? \$_[0] : \join('', @_);
             return _do_send_glob($self, $_to, $_fd, $_data_ref);
          }
@@ -1606,10 +1579,16 @@ sub gather {
 
 sub print {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
-   my $_fd = 0; my ($_glob, $_data_ref);
+   my $self = shift; $self = $MCE unless ref($self);
+   my ($_fd, $_glob, $_data_ref);
 
    if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
+      if (my $_ob = tied *{ $_[0] }) {
+         if (ref $_ob eq 'IO::TieCombine::Handle') {
+            $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
+            $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
+         }
+      }
       $_glob = shift;
    }
 
@@ -1628,10 +1607,16 @@ sub print {
 
 sub printf {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
-   my $_fd = 0; my ($_glob, $_fmt, $_data);
+   my $self = shift; $self = $MCE unless ref($self);
+   my ($_fd, $_glob, $_fmt, $_data);
 
    if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
+      if (my $_ob = tied *{ $_[0] }) {
+         if (ref $_ob eq 'IO::TieCombine::Handle') {
+            $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
+            $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
+         }
+      }
       $_glob = shift;
    }
 
@@ -1645,10 +1630,16 @@ sub printf {
 
 sub say {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
-   my $_fd = 0; my ($_glob, $_data);
+   my $self = shift; $self = $MCE unless ref($self);
+   my ($_fd, $_glob, $_data);
 
    if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
+      if (my $_ob = tied *{ $_[0] }) {
+         if (ref $_ob eq 'IO::TieCombine::Handle') {
+            $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
+            $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
+         }
+      }
       $_glob = shift;
    }
 
@@ -1681,7 +1672,7 @@ sub _croak {
 
 sub _get_max_workers {
 
-   my $x = shift; my $self = ref($x) ? $x : $MCE;
+   my $self = shift; $self = $MCE unless ref($self);
 
    if (defined $self->{user_tasks}) {
       if (defined $self->{user_tasks}->[0]->{max_workers}) {
@@ -1698,21 +1689,22 @@ sub _sync_buffer_to_array {
 
    local $_; my $_cnt = 0;
 
-   open my $_MEM_FILE, '<', $_buffer_ref;
-   binmode $_MEM_FILE;
+   open my $_MEM_FH, '<', $_buffer_ref;
+   binmode $_MEM_FH, ':raw';
 
    unless (length $_chop_str) {
-      $_array_ref->[$_cnt++] = $_ while (<$_MEM_FILE>);
+      $_array_ref->[$_cnt++] = $_ while (<$_MEM_FH>);
    }
    else {
-      $_array_ref->[$_cnt++] = <$_MEM_FILE>;
-      while (<$_MEM_FILE>) {
+      $_array_ref->[$_cnt++] = <$_MEM_FH>;
+      while (<$_MEM_FH>) {
          $_array_ref->[$_cnt  ]  = $_chop_str;
          $_array_ref->[$_cnt++] .= $_;
       }
    }
 
-   close $_MEM_FILE; undef $_MEM_FILE;
+   close  $_MEM_FH;
+   weaken $_MEM_FH;
 
    return;
 }
@@ -1768,13 +1760,13 @@ sub _dispatch {
 
    ## Begin worker.
    $self->{_pid} = ($_is_thread) ? $$ .'.'. threads->tid() : $$;
-   _worker_main(@_args, \@_plugin_worker_init, $_is_MSWin32);
+   _worker_main(@_args, \@_plugin_worker_init);
+
+   ## Check for any Hobo workers not yet joined.
+   MCE::Hobo->finish() if $INC{'MCE/Hobo.pm'};
 
    ## Exit thread/child process.
-   $SIG{__DIE__} = $SIG{__WARN__} = sub {};
-
-   if ($_has_threads && threads->can('exit')) {
-      if ($_is_MSWin32) { lock $_EXT_LOCK; sleep 0.002; }
+   if ($self->{use_threads} || ($_has_threads && $_is_MSWin32)) {
       threads->exit(0);
    }
    elsif ($self->{posix_exit}) {
@@ -1813,7 +1805,7 @@ sub _dispatch_thread {
    if (defined $self->{spawn_delay} && $self->{spawn_delay} > 0.0) {
       sleep $self->{spawn_delay};
    } elsif ($_wid % 4 == 0) {
-      sleep 0.001;
+      sleep 0.001;  # this sleeps for 0.015 on Windows and Cygwin
    }
 
    return;
@@ -1845,7 +1837,7 @@ sub _dispatch_child {
    if (defined $self->{spawn_delay} && $self->{spawn_delay} > 0.0) {
       sleep $self->{spawn_delay};
    } elsif ($_is_winenv && $_wid % 4 == 0) {
-      sleep 0.001;
+      sleep 0.015;
    }
 
    return;
