@@ -14,11 +14,12 @@ package MCE::Core::Input::Iterator;
 use strict;
 use warnings;
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
 no warnings qw( threads recursion uninitialized );
 
