@@ -14,13 +14,15 @@ package MCE::Core::Manager;
 use strict;
 use warnings;
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
+## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
 no warnings qw( threads recursion uninitialized );
 
@@ -90,12 +92,7 @@ sub _output_loop {
       my $_buf = $self->{freeze}($_[0]);
          $_len = length $_buf; local $\ = undef if (defined $\);
 
-      if ($_len < FAST_SEND_SIZE) {
-         print {$_DAU_R_SOCK} $_len . $LF . $_buf;
-      } else {
-         print {$_DAU_R_SOCK} $_len . $LF;
-         print {$_DAU_R_SOCK} $_buf;
-      }
+      print {$_DAU_R_SOCK} $_len.$LF, $_buf;
 
       return;
    };
@@ -105,12 +102,7 @@ sub _output_loop {
       my $_buf = $self->{freeze}($_[0]);
          $_len = length $_buf; local $\ = undef if (defined $\);
 
-      if ($_len < FAST_SEND_SIZE) {
-         print {$_DAU_R_SOCK} WANTS_REF . $LF . $_len . $LF . $_buf;
-      } else {
-         print {$_DAU_R_SOCK} WANTS_REF . $LF . $_len . $LF;
-         print {$_DAU_R_SOCK} $_buf;
-      }
+      print {$_DAU_R_SOCK} WANTS_REF.$LF . $_len.$LF, $_buf;
 
       return;
    };
@@ -120,12 +112,7 @@ sub _output_loop {
       $_len = (defined $_[0]) ? length $_[0] : -1;
       local $\ = undef if (defined $\);
 
-      if ($_len < FAST_SEND_SIZE) {
-         print {$_DAU_R_SOCK} WANTS_SCALAR . $LF . $_len . $LF . $_[0];
-      } else {
-         print {$_DAU_R_SOCK} WANTS_SCALAR . $LF . $_len . $LF;
-         print {$_DAU_R_SOCK} $_[0];
-      }
+      print {$_DAU_R_SOCK} WANTS_SCALAR.$LF . $_len.$LF, $_[0];
 
       return;
    };
@@ -207,7 +194,7 @@ sub _output_loop {
             $self->{_wrk_status} = $_exit_status;
          }
 
-         print {$_DAU_R_SOCK} $LF;
+         print {$_DAU_R_SOCK} $LF unless ($^O eq 'MSWin32');
 
          ## Reap child/thread. Note: Win32 uses negative PIDs.
 
@@ -278,7 +265,8 @@ sub _output_loop {
 
          if ($_offset_pos >= $_input_size || $_aborted) {
             local $\ = undef if (defined $\);
-            print {$_DAU_R_SOCK} '0' . $LF;
+            print {$_DAU_R_SOCK} '0'.$LF;
+
             return;
          }
 
@@ -299,14 +287,7 @@ sub _output_loop {
          }
 
          $_len = length $_buf; local $\ = undef if (defined $\);
-
-         if ($_len < FAST_SEND_SIZE) {
-            print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF . $_buf;
-         } else {
-            print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF;
-            print {$_DAU_R_SOCK} $_buf;
-         }
-
+         print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
          $_offset_pos += $_chunk_size;
 
          return;
@@ -321,7 +302,8 @@ sub _output_loop {
 
          if ($_eof_flag || $_aborted) {
             local $\ = undef if (defined $\);
-            print {$_DAU_R_SOCK} '0' . $LF;
+            print {$_DAU_R_SOCK} '0'.$LF;
+
             return;
          }
 
@@ -360,15 +342,9 @@ sub _output_loop {
          $_len = length $_buf; local $\ = undef if (defined $\);
 
          if ($_len) {
-            if ($_len < FAST_SEND_SIZE) {
-               print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF . $_buf;
-            } else {
-               print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF;
-               print {$_DAU_R_SOCK} $_buf;
-            }
-         }
-         else {
-            print {$_DAU_R_SOCK} '0' . $LF;
+            print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
+         } else {
+            print {$_DAU_R_SOCK} '0'.$LF;
          }
 
          return;
@@ -379,7 +355,8 @@ sub _output_loop {
 
          if ($_aborted) {
             local $\ = undef if (defined $\);
-            print {$_DAU_R_SOCK} '-1' . $LF;
+            print {$_DAU_R_SOCK} '-1'.$LF;
+
             return;
          }
 
@@ -388,33 +365,19 @@ sub _output_loop {
          if (scalar @_ret_a > 1 || ref $_ret_a[0]) {
             $_buf = $self->{freeze}( [ @_ret_a ] );
             $_len = length $_buf; local $\ = undef if (defined $\);
-
-            if ($_len < FAST_SEND_SIZE) {
-               print {$_DAU_R_SOCK}
-                  $_len . '1' . $LF . (++$_chunk_id) . $LF . $_buf;
-            } else {
-               print {$_DAU_R_SOCK} $_len . '1' . $LF . (++$_chunk_id) . $LF;
-               print {$_DAU_R_SOCK} $_buf;
-            }
+            print {$_DAU_R_SOCK} $_len.'1'.$LF . (++$_chunk_id).$LF, $_buf;
 
             return;
          }
          elsif (defined $_ret_a[0]) {
             $_len = length $_ret_a[0]; local $\ = undef if (defined $\);
-
-            if ($_len < FAST_SEND_SIZE) {
-               print {$_DAU_R_SOCK}
-                  $_len . '0' . $LF . (++$_chunk_id) . $LF . $_ret_a[0];
-            } else {
-               print {$_DAU_R_SOCK} $_len . '0' . $LF . (++$_chunk_id) . $LF;
-               print {$_DAU_R_SOCK} $_ret_a[0];
-            }
+            print {$_DAU_R_SOCK} $_len.'0'.$LF . (++$_chunk_id).$LF, $_ret_a[0];
 
             return;
          }
 
          local $\ = undef if (defined $\);
-         print {$_DAU_R_SOCK} '-1' . $LF;
+         print {$_DAU_R_SOCK} '-1'.$LF;
          $_aborted = 1;
 
          return;
@@ -657,7 +620,7 @@ sub _output_loop {
             : $self->{_total_running};
 
          if (++$_sync_cnt == $_total_running) {
-            for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF }
+            for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF; }
             undef $_syn_flag;
          }
 
@@ -671,7 +634,7 @@ sub _output_loop {
                ? $self->{_task}->[0]->{_total_running}
                : $self->{_total_running};
 
-            for (1 .. $_total_running) { 1 until syswrite $_BSE_W_SOCK, $LF }
+            for (1 .. $_total_running) { 1 until syswrite $_BSE_W_SOCK, $LF; }
          }
 
          return;
@@ -754,7 +717,7 @@ sub _output_loop {
       binmode $_MCE_STDOUT;
    }
    else {
-      open $_MCE_STDOUT, '>&=STDOUT';
+      $_MCE_STDOUT = \*STDOUT;
       binmode $_MCE_STDOUT;
    }
 
@@ -764,7 +727,7 @@ sub _output_loop {
       binmode $_MCE_STDERR;
    }
    else {
-      open $_MCE_STDERR, '>&=STDERR';
+      $_MCE_STDERR = \*STDERR;
       binmode $_MCE_STDERR;
    }
 
@@ -828,7 +791,7 @@ sub _output_loop {
          print {$_DAT_W_SOCK} 'NOOP'.$LF . '0'.$LF;
       };
 
-      while (1) {
+      while ( $self->{_total_running} ) {
          alarm $_timeout;
          $_func = <$_DAT_R_SOCK>;
          $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
@@ -839,16 +802,53 @@ sub _output_loop {
          } elsif (exists $_plugin_function->{$_func}) {
             $_plugin_function->{$_func}();
          }
+      }
+   }
+
+   ## Wait on requests *without* timeout capability.
+
+   elsif ($^O eq 'MSWin32') {
+      # The normal loop hangs on Windows when processes/threads start/exit.
+      # Using ioctl() properly, http://www.perlmonks.org/?node_id=780083
+
+      my ($_bytes, $_cnt, $_retries) = ("\x00\x00\x00\x00", 1, 0);
+      my ($_ptr_bytes, $_nbytes) = (unpack('I', pack('P', $_bytes)));
+
+      while (1) {
+         ioctl($_DAT_R_SOCK, 0x4004667f, $_ptr_bytes);  # MSWin32 FIONREAD
+
+         unless ($_nbytes = unpack('I', $_bytes)) {
+            # delay to not consume a CPU from non-blocking ioctl
+            if ($_cnt) {
+               if (++$_cnt > 900) {
+                  $_cnt = 1, sleep 0.015;
+                  $_cnt = 0 if ++$_retries == 2;
+               }
+               next;
+            }
+            sleep 0.030;
+            next;
+         }
+
+         $_cnt = 1, $_retries = 0;
+
+         do {
+            sysread($_DAT_R_SOCK, $_func, 8);
+            $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
+
+            if (exists $_core_output_function{$_func}) {
+               $_core_output_function{$_func}();
+            } elsif (exists $_plugin_function->{$_func}) {
+               $_plugin_function->{$_func}();
+            }
+
+         } while (($_nbytes -= 8) >= 8);
 
          last unless $self->{_total_running};
       }
    }
-
-   ## Otherwise, wait on requests *without* timeout capability.
-   ## Exit loop when all workers have completed processing.
-
    else {
-      while (1) {
+      while ($self->{_total_running}) {
          $_func = <$_DAT_R_SOCK>;
          $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
 
@@ -857,8 +857,6 @@ sub _output_loop {
          } elsif (exists $_plugin_function->{$_func}) {
             $_plugin_function->{$_func}();
          }
-
-         last unless $self->{_total_running};
       }
    }
 
@@ -884,8 +882,10 @@ sub _output_loop {
 
    select $_old_hndl;
 
-   close $_MCE_STDOUT; undef $_MCE_STDOUT;
-   close $_MCE_STDERR; undef $_MCE_STDERR;
+   eval q{
+      close $_MCE_STDOUT if (fileno $_MCE_STDOUT > 2);
+      close $_MCE_STDERR if (fileno $_MCE_STDERR > 2);
+   };
 
    return;
 }
