@@ -14,11 +14,12 @@ package MCE::Core::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
 no warnings qw( threads recursion uninitialized );
 
@@ -33,7 +34,7 @@ use bytes;
 
 {
    my (
-      $_dest, $_len, $_tag, $_task_id, $_user_func, $_value, $_want_id,
+      $_dest, $_len, $_tag, $_task_id, $_user_func, $_val, $_wa,
       $_DAT_LOCK, $_DAT_W_SOCK, $_DAU_W_SOCK, $_chn, $_lock_chn,
       $_dat_ex, $_dat_un
    );
@@ -43,14 +44,13 @@ use bytes;
 
    $_dest_function[SENDTO_FILEV2] = sub {         ## Content >> File
 
-      return unless (defined $_value);
+      return unless (defined $_val);
       local $\ = undef if (defined $\);
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_F_SND . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} $_value . $LF . length(${ $_[0] }) . $LF;
-         print {$_DAU_W_SOCK} ${ $_[0] };
+         print {$_DAT_W_SOCK} OUTPUT_F_SND.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} $_val.$LF . length(${ $_[0] }).$LF, ${ $_[0] };
          $_dat_un->() if $_lock_chn;
       }
 
@@ -59,14 +59,13 @@ use bytes;
 
    $_dest_function[SENDTO_FD] = sub {             ## Content >> File descriptor
 
-      return unless (defined $_value);
+      return unless (defined $_val);
       local $\ = undef if (defined $\);
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_D_SND . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} $_value . $LF . length(${ $_[0] }) . $LF;
-         print {$_DAU_W_SOCK} ${ $_[0] };
+         print {$_DAT_W_SOCK} OUTPUT_D_SND.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} $_val.$LF . length(${ $_[0] }).$LF, ${ $_[0] };
          $_dat_un->() if $_lock_chn;
       }
 
@@ -79,9 +78,8 @@ use bytes;
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_O_SND . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} length(${ $_[0] }) . $LF;
-         print {$_DAU_W_SOCK} ${ $_[0] };
+         print {$_DAT_W_SOCK} OUTPUT_O_SND.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} length(${ $_[0] }).$LF, ${ $_[0] };
          $_dat_un->() if $_lock_chn;
       }
 
@@ -94,9 +92,8 @@ use bytes;
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_E_SND . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} length(${ $_[0] }) . $LF;
-         print {$_DAU_W_SOCK} ${ $_[0] };
+         print {$_DAT_W_SOCK} OUTPUT_E_SND.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} length(${ $_[0] }).$LF, ${ $_[0] };
          $_dat_un->() if $_lock_chn;
       }
 
@@ -107,14 +104,14 @@ use bytes;
 
    sub _do_callback {
 
-      my ($self, $_buf, $_aref);  ($self, $_value, $_aref) = @_;
+      my ($self, $_buf, $_aref);  ($self, $_val, $_aref) = @_;
 
       unless (defined wantarray) {
-         $_want_id = WANTS_UNDEF;
+         $_wa = WANTS_UNDEF;
       } elsif (wantarray) {
-         $_want_id = WANTS_ARRAY;
+         $_wa = WANTS_ARRAY;
       } else {
-         $_want_id = WANTS_SCALAR;
+         $_wa = WANTS_SCALAR;
       }
 
       ## Crossover: Send arguments
@@ -126,9 +123,8 @@ use bytes;
             $_len = length $_buf; local $\ = undef if (defined $\);
 
             $_dat_ex->() if $_lock_chn;
-            print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
-            print {$_DAU_W_SOCK} $_want_id . $LF . $_value . $LF . $_len . $LF;
-            print {$_DAU_W_SOCK} $_buf;
+            print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
+            print {$_DAU_W_SOCK} $_wa.$LF . $_val.$LF . $_len.$LF, $_buf;
 
          }
          else {                                   ## Scalar >> Callback
@@ -136,9 +132,8 @@ use bytes;
             $_len = length $_aref->[0]; local $\ = undef if (defined $\);
 
             $_dat_ex->() if $_lock_chn;
-            print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
-            print {$_DAU_W_SOCK} $_want_id . $LF . $_value . $LF . $_len . $LF;
-            print {$_DAU_W_SOCK} $_aref->[0];
+            print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
+            print {$_DAU_W_SOCK} $_wa.$LF . $_val.$LF . $_len.$LF, $_aref->[0];
          }
       }
       else {                                      ## No Args >> Callback
@@ -146,17 +141,17 @@ use bytes;
          local $\ = undef if (defined $\);
 
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
-         print {$_DAU_W_SOCK} $_want_id . $LF . $_value . $LF;
+         print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
+         print {$_DAU_W_SOCK} $_wa.$LF . $_val.$LF;
       }
 
       ## Crossover: Receive return value
 
-      if ($_want_id == WANTS_UNDEF) {
+      if ($_wa == WANTS_UNDEF) {
          $_dat_un->() if $_lock_chn;
          return;
       }
-      elsif ($_want_id == WANTS_ARRAY) {
+      elsif ($_wa == WANTS_ARRAY) {
          local $/ = $LF if (!$/ || $/ ne $LF);
          chomp($_len = <$_DAU_W_SOCK>);
 
@@ -167,14 +162,14 @@ use bytes;
       }
       else {
          local $/ = $LF if (!$/ || $/ ne $LF);
-         chomp($_want_id = <$_DAU_W_SOCK>);
+         chomp($_wa = <$_DAU_W_SOCK>);
          chomp($_len     = <$_DAU_W_SOCK>);
 
          if ($_len >= 0) {
             read($_DAU_W_SOCK, $_buf, $_len || 0);
             $_dat_un->() if $_lock_chn;
 
-            return $_buf if ($_want_id == WANTS_SCALAR);
+            return $_buf if ($_wa == WANTS_SCALAR);
             return $self->{thaw}($_buf);
          }
          else {
@@ -203,9 +198,8 @@ use bytes;
             $_len = length $_aref->[0]; local $\ = undef if (defined $\);
 
             $_dat_ex->() if $_lock_chn;
-            print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
-            print {$_DAU_W_SOCK} $_task_id . $LF . $_len . $LF;
-            print {$_DAU_W_SOCK} $_aref->[0];
+            print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
+            print {$_DAU_W_SOCK} $_task_id.$LF . $_len.$LF, $_aref->[0];
             $_dat_un->() if $_lock_chn;
 
             return;
@@ -219,8 +213,8 @@ use bytes;
       local $\ = undef if (defined $\);
 
       $_dat_ex->() if $_lock_chn;
-      print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
-      print {$_DAU_W_SOCK} $_task_id . $LF . $_len . $LF, $_buf;
+      print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
+      print {$_DAU_W_SOCK} $_task_id.$LF . $_len.$LF, $_buf;
       $_dat_un->() if $_lock_chn;
 
       return;
@@ -232,7 +226,7 @@ use bytes;
 
       my $_data_ref; my $self = shift;
 
-      $_dest = shift; $_value = shift;
+      $_dest = shift; $_val = shift;
 
       if (scalar @_ > 1) {
          $_data_ref = \join('', @_);
@@ -313,7 +307,7 @@ use bytes;
 
       my ($self) = @_;
 
-      $_dest = $_len = $_task_id = $_user_func = $_value = $_want_id = undef;
+      $_dest = $_len = $_task_id = $_user_func = $_val = $_wa = undef;
       $_DAT_LOCK = $_DAT_W_SOCK = $_DAU_W_SOCK = $_chn = $_lock_chn = undef;
       $_dat_ex = $_dat_un = $_tag = undef;
 
@@ -358,7 +352,6 @@ sub _worker_do {
    @_ = ();
 
    ## Set options.
-   $self->{_running}    = 1;
    $self->{_abort_msg}  = $_params_ref->{_abort_msg};
    $self->{_run_mode}   = $_params_ref->{_run_mode};
    $self->{_single_dim} = $_params_ref->{_single_dim};
@@ -472,17 +465,15 @@ sub _worker_do {
    $_DAT_LOCK->lock() if $_lock_chn;
 
    if (exists $self->{_rla_return}) {
-      print {$_DAT_W_SOCK} OUTPUT_W_RLA . $LF . $_chn . $LF;
-      print {$_DAU_W_SOCK} (delete $self->{_rla_return}) . $LF;
+      print {$_DAT_W_SOCK} OUTPUT_W_RLA.$LF . $_chn.$LF;
+      print {$_DAU_W_SOCK} (delete $self->{_rla_return}).$LF;
    }
 
-   print {$_DAT_W_SOCK} OUTPUT_W_DNE . $LF . $_chn . $LF;
-   print {$_DAU_W_SOCK} $_task_id . $LF;
+   print {$_DAT_W_SOCK} OUTPUT_W_DNE.$LF . $_chn.$LF;
+   print {$_DAU_W_SOCK} $_task_id.$LF;
 
    $_DAT_LOCK->unlock() if $_lock_chn;
 
-   $self->{_running} = 0;
-
    return;
 }
 
@@ -509,6 +500,7 @@ sub _worker_loop {
    my $_com_un = sub { 1 until syswrite( $_COM_LOCK->{_w_sock}, '0' ) };
 
    while (1) {
+      MCE::Util::_sock_ready($_COM_W_SOCK) if ($^O eq 'MSWin32');
 
       {
          local $\ = undef; local $/ = $LF;
@@ -516,7 +508,7 @@ sub _worker_loop {
 
          ## Wait for the next job request.
          $_response = <$_COM_W_SOCK>;
-         print {$_COM_W_SOCK} $_wid . $LF;
+         print {$_COM_W_SOCK} $_wid.$LF;
 
          ## End loop if invalid response.
          last unless (defined $_response);
@@ -535,7 +527,7 @@ sub _worker_loop {
             chomp($_len = <$_COM_W_SOCK>);
             read $_COM_W_SOCK, $_buf, $_len;
 
-            print {$_COM_W_SOCK} $_wid . $LF;
+            print {$_COM_W_SOCK} $_wid.$LF;
             $_com_un->();
 
             $self->{user_data} = $self->{thaw}($_buf);
@@ -553,7 +545,7 @@ sub _worker_loop {
             chomp($_len = <$_COM_W_SOCK>);
             read $_COM_W_SOCK, $_buf, $_len;
 
-            print {$_COM_W_SOCK} $_wid . $LF;
+            print {$_COM_W_SOCK} $_wid.$LF;
             $_com_un->();
 
             $_params_ref = $self->{thaw}($_buf);
@@ -592,7 +584,7 @@ sub _worker_loop {
 sub _worker_main {
 
    my ( $self, $_wid, $_task, $_task_id, $_task_wid, $_params,
-        $_plugin_worker_init, $_is_MSWin32 ) = @_;
+        $_plugin_worker_init ) = @_;
 
    @_ = ();
 
@@ -605,7 +597,6 @@ sub _worker_main {
    $self->{_task_wid} = (defined $_task_wid) ? $_task_wid : $_wid;
    $self->{_task}     = $_task;
    $self->{_wid}      = $_wid;
-   $self->{_running}  = 0;
 
    ## Define exit pid and DIE handler.
    my $_use_threads = (defined $_task->{use_threads})
@@ -636,7 +627,7 @@ sub _worker_main {
          }
       }
 
-      local $SIG{__DIE__} = sub {}; local $\ = undef;
+      local $SIG{__DIE__}; local $\ = undef;
       my $_die_msg = (defined $_[0]) ? $_[0] : '';
       print {*STDERR} $_die_msg;
 
@@ -666,16 +657,6 @@ sub _worker_main {
       $_chn = $self->{_chn} = $_wid % $self->{_data_channels} + 1;
    }
 
-   ## Unset the need for channel locking if only worker on the channel.
-   if ($self->{_lock_chn} && !exists $INC{'MCE/Hobo.pm'}) {
-      my $_data_channels = $self->{_data_channels};
-      if ($self->{_init_total_workers} < $_data_channels * 2) {
-         if ($_wid > $self->{_init_total_workers} % $_data_channels) {
-            $self->{_lock_chn} = 0 if ($_wid <= $_data_channels);
-         }
-      }
-   }
-
    ## Choose locks for DATA channels.
    $self->{_com_lock} = $self->{'_mutex_0'};
    $self->{_dat_lock} = $self->{'_mutex_'.$_chn} if ($self->{_lock_chn});
@@ -687,8 +668,6 @@ sub _worker_main {
       _pids _state _status _thrs _tids
    ) };
 
-   MCE::_clean_sessions($_mce_sid);
-
    ## Call MCE::Shared's init routine if present; enables parallel IPC.
    MCE::Shared::init($_wid) if ($INC{'MCE/Shared.pm'});
 
@@ -700,12 +679,9 @@ sub _worker_main {
    ## Begin processing if worker was added during processing. Otherwise,
    ## respond back to the main process if the last worker spawned.
    if (defined $_params) {
-      sleep 0.002; _worker_do($self, $_params);
+      _worker_do($self, $_params);
       undef $_params;
    }
-   elsif ($_is_MSWin32) {
-      lock $MCE::_WIN_LOCK;
-   }
 
    ## Enter worker loop.
    _worker_loop($self);
@@ -716,8 +692,6 @@ sub _worker_main {
    $self->{_com_lock} = undef;
    $self->{_dat_lock} = undef;
 
-   MCE::_clear_session($_mce_sid);
-
    return;
 }
 
