@@ -11,18 +11,15 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
 use MCE::Util qw( $LF );
 
-our @CARP_NOT = qw( MCE::Shared MCE );
-
 my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
 my $_tid = $_has_threads ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid();
-   return;
+   $_tid = threads->tid() if $_has_threads;
 }
 
 sub DESTROY {
@@ -31,10 +28,8 @@ sub DESTROY {
 
    $_obj->unlock() if ($_obj->{ $_pid });
 
-   if ($_arg eq 'shutdown' || $_obj->{'init_pid'} eq $_pid) {
-      ($^O eq 'MSWin32')
-         ? MCE::Util::_destroy_pipes($_obj, qw(_w_sock _r_sock))
-         : MCE::Util::_destroy_socks($_obj, qw(_w_sock _r_sock));
+   if ($_obj->{'init_pid'} eq $_pid || $_arg eq 'shutdown') {
+      MCE::Util::_destroy_socks($_obj, qw(_w_sock _r_sock));
    }
 
    return;
@@ -47,17 +42,13 @@ sub DESTROY {
 ###############################################################################
 
 sub new {
-   my ($_class, %_argv) = @_; my $_obj = { %_argv };
+   my ($_class, %_obj) = @_;
+   $_obj{'init_pid'} = $_has_threads ? $$ .'.'. $_tid : $$;
 
-   $_obj->{'init_pid'} = $_has_threads ? $$ .'.'. $_tid : $$;
+   MCE::Util::_sock_pair(\%_obj, qw(_r_sock _w_sock));
+   1 until syswrite($_obj{_w_sock}, '0');
 
-   ($^O eq 'MSWin32')
-      ? MCE::Util::_pipe_pair($_obj, qw(_r_sock _w_sock))
-      : MCE::Util::_sock_pair($_obj, qw(_r_sock _w_sock));
-
-   1 until syswrite($_obj->{_w_sock}, '0');
-
-   return bless($_obj, $_class);
+   return bless(\%_obj, $_class);
 }
 
 sub lock {
@@ -121,7 +112,7 @@ MCE::Mutex - Locking for Many-Core Engine
 
 =head1 VERSION
 
-This document describes MCE::Mutex version 1.699_013
+This document describes MCE::Mutex version 1.799_01
 
 =head1 SYNOPSIS
 
@@ -194,7 +185,7 @@ completes. Optionally, the method is C<wantarray> aware.
 
 =head1 INDEX
 
-L<MCE|MCE>, L<MCE::Core>, L<MCE::Shared>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
