@@ -11,92 +11,85 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use Scalar::Util qw( looks_like_number );
-use Storable ();
-use MCE::Signal;
+use MCE;
+use MCE::Queue;
 
 our @CARP_NOT = qw( MCE );
 
+my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $_tid = $_has_threads ? threads->tid() : 0;
+my $_caller;
+
+sub CLONE {
+   $_tid = threads->tid() if $_has_threads;
+}
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Import routine.
 ##
 ###############################################################################
 
-my ($MAX_WORKERS, $CHUNK_SIZE) = ('auto', 'auto');
-
-my $TMP_DIR = $MCE::Signal::tmp_dir;
-my $FREEZE  = \&Storable::freeze;
-my $THAW    = \&Storable::thaw;
-
-my ($DEFAULT_MODE, $FAST) = ('map', 0);
-
-my ($_params, @_prev_c, @_prev_m, @_prev_n, @_prev_w, @_user_tasks, @_queue);
-my ($_MCE, $_imported); my $_tag = 'MCE::Stream';
+my ($_MCE, $_def, $_params, $_tag) = ({}, {}, {}, 'MCE::Stream');
+my ($_prev_c, $_prev_m, $_prev_n, $_prev_w) = ({}, {}, {}, {});
+my ($_user_tasks, $_queue) = ({}, {});
 
 sub import {
-   my $_class = shift; return if ($_imported++);
+   my ($_class, $_pkg) = (shift, caller);
 
-   ## Process module arguments.
-   while (my $_argument = shift) {
-      my $_arg = lc $_argument;
-
-      $MAX_WORKERS  = shift, next if ( $_arg eq 'max_workers' );
-      $CHUNK_SIZE   = shift, next if ( $_arg eq 'chunk_size' );
-      $FREEZE       = shift, next if ( $_arg eq 'freeze' );
-      $THAW         = shift, next if ( $_arg eq 'thaw' );
-      $TMP_DIR      = shift, next if ( $_arg eq 'tmp_dir' );
-      $DEFAULT_MODE = shift, next if ( $_arg eq 'default_mode' );
-      $FAST         = shift, next if ( $_arg eq 'fast' );
-
-      if ( $_arg eq 'sereal' ) {
-         if (shift eq '1') {
-            local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            $FREEZE = \&encode_sereal, $THAW = \&decode_sereal unless $@;
-         }
-         next;
-      }
-
-      _croak("Error: ($_argument) invalid module option");
-   }
-
-   _croak("Error: (DEFAULT_MODE) is not valid")
-      if ($DEFAULT_MODE ne 'grep' && $DEFAULT_MODE ne 'map');
-
-   ## Preload essential modules.
-   require MCE; MCE->import(
-      freeze => $FREEZE, thaw => $THAW, tmp_dir => $TMP_DIR
-   );
-
-   $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
-   _validate_number($MAX_WORKERS, 'MAX_WORKERS');
-
-   _validate_number($CHUNK_SIZE, 'CHUNK_SIZE')
-      unless ($CHUNK_SIZE eq 'auto');
-
-   require MCE::Queue; MCE::Queue->import();
+   my $_p = $_def->{$_pkg} = {
+      MAX_WORKERS  => 'auto',
+      CHUNK_SIZE   => 'auto',
+      DEFAULT_MODE => 'map',
+      FAST         => 0,
+   };
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_pkg = caller;
 
    *{ $_pkg.'::mce_stream_f' } = \&run_file;
    *{ $_pkg.'::mce_stream_s' } = \&run_seq;
    *{ $_pkg.'::mce_stream'   } = \&run;
 
+   ## Process module arguments.
+   while ( my $_argument = shift ) {
+      my $_arg = lc $_argument;
+
+      $_p->{MAX_WORKERS}  = shift, next if ( $_arg eq 'max_workers' );
+      $_p->{CHUNK_SIZE}   = shift, next if ( $_arg eq 'chunk_size' );
+      $_p->{TMP_DIR}      = shift, next if ( $_arg eq 'tmp_dir' );
+      $_p->{FREEZE}       = shift, next if ( $_arg eq 'freeze' );
+      $_p->{THAW}         = shift, next if ( $_arg eq 'thaw' );
+      $_p->{DEFAULT_MODE} = shift, next if ( $_arg eq 'default_mode' );
+      $_p->{FAST}         = shift, next if ( $_arg eq 'fast' );
+
+      ## Sereal, if available, is used automatically by MCE 1.800 onwards.
+      next if ( $_arg eq 'sereal' );
+
+      _croak("Error: ($_argument) invalid module option");
+   }
+
+   _croak("Error: (DEFAULT_MODE) is not valid")
+      if ($_p->{DEFAULT_MODE} ne 'grep' && $_p->{DEFAULT_MODE} ne 'map');
+
+   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+
+   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
+   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+      unless ($_p->{CHUNK_SIZE} eq 'auto');
+
    return;
 }
 
 END {
-   return if (defined $_MCE && $_MCE->wid);
-
-   finish();
+   $_params = $_prev_c = $_queue = $_user_tasks = $_MCE = undef;
 }
 
 ###############################################################################
@@ -128,12 +121,13 @@ sub _preserve_order {
 sub _task_end {
 
    my ($_mce, $_task_id, $_task_name) = @_;
+   my $_pid = $_mce->{_init_pid}.'.'.$_mce->{_caller};
 
    if (defined $_mce->{user_tasks}->[$_task_id + 1]) {
       my $n_workers = $_mce->{user_tasks}->[$_task_id + 1]->{max_workers};
-      my $_queue_id = @_queue - $_task_id - 1;
+      my $_id = @{ $_queue->{$_pid} } - $_task_id - 1;
 
-      $_queue[$_queue_id]->enqueue((undef) x $n_workers);
+      $_queue->{$_pid}[$_id]->enqueue((undef) x $n_workers);
    }
 
    $_params->{task_end}->($_mce, $_task_id, $_task_name)
@@ -152,28 +146,39 @@ sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
-   if (MCE->wid) {
-      @_ = (); _croak("$_tag: (init) is not allowed by the worker process");
-   }
+   my $_pid = "$$.$_tid.".caller();
 
-   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
+   finish(); $_params->{$_pid} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
    @_ = ();
 
    return;
 }
 
-sub finish () {
+sub finish (@) {
 
-   if (defined $_MCE && $_MCE->{_spawned}) {
-      MCE::_save_state(); $_MCE->shutdown(); MCE::_restore_state();
+   shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
+
+   my $_pid = "$$.$_tid.".caller();
+
+   if (defined $_MCE->{$_pid}) {
+      MCE::_save_state(), $_MCE->{$_pid}->shutdown(@_), MCE::_restore_state()
+         if $_MCE->{$_pid}->{_spawned};
+
+      $_gather_ref = $_order_id = undef; undef %_tmp;
+
+      delete $_user_tasks->{$_pid};
+      delete $_prev_c->{$_pid};
+      delete $_prev_m->{$_pid};
+      delete $_prev_n->{$_pid};
+      delete $_prev_w->{$_pid};
+
+      if (defined $_queue->{$_pid}) {
+         $_->DESTROY() for (@{ $_queue->{$_pid} });
+         delete $_queue->{$_pid};
+      }
    }
 
-   $_gather_ref = $_order_id = undef; undef %_tmp; @_user_tasks = ();
-   @_prev_w = (); @_prev_n = (); @_prev_m = (); @_prev_c = ();
-
-   $_->DESTROY() for (@_queue); @_queue = ();
-
    return;
 }
 
@@ -188,13 +193,14 @@ sub run_file (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    my ($_file, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
+   my $_pid = "$$.$_tid.".caller();  $_caller = caller();
 
-   if (defined $_params) {
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{sequence}   if (exists $_params->{sequence});
+   if (defined (my $_p = $_params->{$_pid})) {
+      delete $_p->{input_data} if (exists $_p->{input_data});
+      delete $_p->{sequence}   if (exists $_p->{sequence});
    }
    else {
-      $_params = {};
+      $_params->{$_pid} = {};
    }
 
    for my $_i ($_start_pos .. @_ - 1) {
@@ -206,13 +212,13 @@ sub run_file (@) {
    }
 
    if (defined $_file && ref $_file eq '' && $_file ne '') {
-      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
-      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) does not exist")      unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable")     unless (-r $_file);
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
-      $_params->{_file} = $_file;
+      $_params->{$_pid}{_file} = $_file;
    }
    elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
-      $_params->{_file} = $_file;
+      $_params->{$_pid}{_file} = $_file;
    }
    else {
       _croak("$_tag: (file) is not specified or valid");
@@ -236,14 +242,15 @@ sub run_seq (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    my ($_begin, $_end, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
+   my $_pid = "$$.$_tid.".caller();  $_caller = caller();
 
-   if (defined $_params) {
-      delete $_params->{sequence}   if (exists $_params->{sequence});
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{_file}      if (exists $_params->{_file});
+   if (defined (my $_p = $_params->{$_pid})) {
+      delete $_p->{sequence}   if (exists $_p->{sequence});
+      delete $_p->{input_data} if (exists $_p->{input_data});
+      delete $_p->{_file}      if (exists $_p->{_file});
    }
    else {
-      $_params = {};
+      $_params->{$_pid} = {};
    }
 
    for my $_i ($_start_pos .. @_ - 1) {
@@ -254,17 +261,17 @@ sub run_seq (@) {
 
          if ($_ref eq '') {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
-            $_params->{sequence} = [
+            $_params->{$_pid}{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
             ];
          }
          elsif ($_ref eq 'HASH') {
             $_begin = $_[$_pos]->{begin}; $_end = $_[$_pos]->{end};
-            $_params->{sequence} = $_[$_pos];
+            $_params->{$_pid}{sequence} = $_[$_pos];
          }
          elsif ($_ref eq 'ARRAY') {
             $_begin = $_[$_pos]->[0]; $_end = $_[$_pos]->[1];
-            $_params->{sequence} = $_[$_pos];
+            $_params->{$_pid}{sequence} = $_[$_pos];
          }
 
          last;
@@ -272,15 +279,13 @@ sub run_seq (@) {
    }
 
    _croak("$_tag: (sequence) is not specified or valid")
-      unless (exists $_params->{sequence});
-
+      unless (exists $_params->{$_pid}{sequence});
    _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
-
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = 1;
 
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
@@ -299,14 +304,13 @@ sub run (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
-   if (MCE->wid) {
-      @_ = (); _croak("$_tag: (run) is not allowed by the worker process");
-   }
+   my $_pkg = defined $_caller ? $_caller : caller();  $_caller = undef;
+   my $_pid = "$$.$_tid.$_pkg";
 
    if (ref $_[0] eq 'HASH' && !exists $_[0]->{code}) {
-      $_params = {} unless defined $_params;
+      $_params->{$_pid} = {} unless defined $_params->{$_pid};
       for my $_p (keys %{ $_[0] }) {
-         $_params->{$_p} = $_[0]->{$_p};
+         $_params->{$_pid}{$_p} = $_[0]->{$_p};
       }
 
       shift;
@@ -318,20 +322,23 @@ sub run (@) {
 
    if (defined $_aref) {
       $_gather_ref = $_aref; @{ $_aref } = ();
+   } else {
+      $_gather_ref = undef;
    }
 
    ## -------------------------------------------------------------------------
 
    my (@_code, @_mode, @_name, @_wrks); my $_init_mce = 0; my $_pos = 0;
+   my $_default_mode = $_def->{$_pkg}{DEFAULT_MODE};
 
    while (ref $_[0] eq 'CODE' || ref $_[0] eq 'HASH') {
       if (ref $_[0] eq 'CODE') {
          push @_code, $_[0];
-         push @_mode, $DEFAULT_MODE;
+         push @_mode, $_default_mode;
       }
       else {
          push @_code, exists $_[0]->{code} ? $_[0]->{code} : undef;
-         push @_mode, exists $_[0]->{mode} ? $_[0]->{mode} : $DEFAULT_MODE;
+         push @_mode, exists $_[0]->{mode} ? $_[0]->{mode} : $_default_mode;
 
          unless (ref $_code[-1] eq 'CODE') {
             @_ = (); _croak("$_tag: (code) is not valid");
@@ -341,35 +348,38 @@ sub run (@) {
          }
       }
 
-      push @_name, (defined $_params && ref $_params->{task_name} eq 'ARRAY')
-         ? $_params->{task_name}->[$_pos] : undef;
-      push @_wrks, (defined $_params && ref $_params->{max_workers} eq 'ARRAY')
-         ? $_params->{max_workers}->[$_pos] : undef;
-
-      $_init_mce = 1
-         if (!defined $_prev_c[$_pos] || $_prev_c[$_pos] != $_code[$_pos]);
-      $_init_mce = 1
-         if (!defined $_prev_m[$_pos] || $_prev_m[$_pos] ne $_mode[$_pos]);
-
-      {
-         no warnings;
-         $_init_mce = 1 if ($_prev_n[$_pos] ne $_name[$_pos]);
-         $_init_mce = 1 if ($_prev_w[$_pos] ne $_wrks[$_pos]);
+      if (defined (my $_p = $_params->{$_pid})) {
+         push @_name, (ref $_p->{task_name} eq 'ARRAY')
+            ? $_p->{task_name}->[$_pos] : undef;
+         push @_wrks, (ref $_p->{max_workers} eq 'ARRAY')
+            ? $_p->{max_workers}->[$_pos] : undef;
       }
 
-      $_prev_c[$_pos] = $_code[$_pos];
-      $_prev_m[$_pos] = $_mode[$_pos];
-      $_prev_n[$_pos] = $_name[$_pos];
-      $_prev_w[$_pos] = $_wrks[$_pos];
+      $_init_mce = 1 if (
+         !defined $_prev_c->{$_pid}[$_pos] ||
+         $_prev_c->{$_pid}[$_pos] != $_code[$_pos]
+      );
+      $_init_mce = 1 if (
+         !defined $_prev_m->{$_pid}[$_pos] ||
+         $_prev_m->{$_pid}[$_pos] ne $_mode[$_pos]
+      );
+
+      $_init_mce = 1 if ($_prev_n->{$_pid}[$_pos] ne $_name[$_pos]);
+      $_init_mce = 1 if ($_prev_w->{$_pid}[$_pos] ne $_wrks[$_pos]);
+
+      $_prev_c->{$_pid}[$_pos] = $_code[$_pos];
+      $_prev_m->{$_pid}[$_pos] = $_mode[$_pos];
+      $_prev_n->{$_pid}[$_pos] = $_name[$_pos];
+      $_prev_w->{$_pid}[$_pos] = $_wrks[$_pos];
 
       shift; $_pos++;
    }
 
-   if (defined $_prev_c[$_pos]) {
-      pop @_prev_c for ($_pos .. @_prev_c - 1);
-      pop @_prev_m for ($_pos .. @_prev_m - 1);
-      pop @_prev_n for ($_pos .. @_prev_n - 1);
-      pop @_prev_w for ($_pos .. @_prev_w - 1);
+   if (defined $_prev_c->{$_pid}[$_pos]) {
+      pop @{ $_prev_c->{$_pid} } for ($_pos .. $#{ $_prev_c->{$_pid } });
+      pop @{ $_prev_m->{$_pid} } for ($_pos .. $#{ $_prev_m->{$_pid } });
+      pop @{ $_prev_n->{$_pid} } for ($_pos .. $#{ $_prev_n->{$_pid } });
+      pop @{ $_prev_w->{$_pid} } for ($_pos .. $#{ $_prev_w->{$_pid } });
 
       $_init_mce = 1;
    }
@@ -378,13 +388,14 @@ sub run (@) {
 
    ## -------------------------------------------------------------------------
 
-   my $_input_data; my $_max_workers = $MAX_WORKERS; my $_r = ref $_[0];
+   my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
+   my $_r = ref $_[0];
 
    if ($_r eq 'ARRAY' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
       $_input_data = shift;
    }
 
-   if (defined $_params) { my $_p = $_params;
+   if (defined (my $_p = $_params->{$_pid})) {
       $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers} && ref $_p->{max_workers} ne 'ARRAY');
 
@@ -401,40 +412,42 @@ sub run (@) {
    }
 
    my $_chunk_size = MCE::Util::_parse_chunk_size(
-      $CHUNK_SIZE, $_max_workers, $_params, $_input_data, scalar @_
+      $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
+      $_input_data, scalar @_
    );
 
-   if (defined $_params) {
-      if (exists $_params->{_file}) {
-         $_input_data = delete $_params->{_file};
-      }
-      else {
-         $_input_data = $_params->{input_data} if exists $_params->{input_data};
+   if (defined (my $_p = $_params->{$_pid})) {
+      if (exists $_p->{_file}) {
+         $_input_data = delete $_p->{_file};
+      } else {
+         $_input_data = $_p->{input_data} if exists $_p->{input_data};
       }
    }
 
-   MCE::_save_state();
-
    ## -------------------------------------------------------------------------
 
+   MCE::_save_state();
+
    if ($_init_mce) {
-      $_MCE->shutdown() if (defined $_MCE);
+      $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
+      $_queue->{$_pid} = [] if (!defined $_queue->{$_pid});
 
-      pop( @_queue )->DESTROY for (@_code .. @_queue);
+      my $_Q = $_queue->{$_pid};
+      pop(@{ $_Q })->DESTROY for (@_code .. @{ $_Q });
 
-      push @_queue, MCE::Queue->new(fast => $FAST)
-         for (@_queue .. @_code - 2);
+      push @{ $_Q }, MCE::Queue->new(fast => $_def->{$_pkg}{FAST})
+         for (@{ $_Q } .. @_code - 2);
 
-      _gen_user_tasks(\@_queue, \@_code, \@_mode, \@_name, \@_wrks);
+      _gen_user_tasks($_pid, $_Q, \@_code, \@_mode, \@_name, \@_wrks);
 
-      my %_options = (
+      my %_opts = (
          max_workers => $_max_workers, task_name => $_tag,
-         user_tasks => \@_user_tasks, task_end => \&_task_end,
+         user_tasks  => $_user_tasks->{$_pid}, task_end => \&_task_end,
          use_slurpio => 0,
       );
 
-      if (defined $_params) {
-         local $_; my $_p = $_params;
+      if (defined (my $_p = $_params->{$_pid})) {
+         local $_;
 
          for (keys %{ $_p }) {
             next if ($_ eq 'sequence_run');
@@ -447,22 +460,27 @@ sub run (@) {
             _croak("MCE::Stream: ($_) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_});
 
-            $_options{$_} = $_p->{$_};
+            $_opts{$_} = $_p->{$_};
          }
       }
 
-      $_MCE = MCE->new(%_options);
+      for my $_k (qw/ tmp_dir freeze thaw /) {
+         $_opts{$_k} = $_def->{$_pkg}{uc($_k)}
+            if (exists $_def->{$_pkg}{uc($_k)} && !exists $_opts{$_k});
+      }
+
+      $_MCE->{$_pid} = MCE->new(pkg => $_pkg, %_opts);
    }
    else {
       ## Workers may persist after running. Thus, updating the MCE instance.
       ## These options do not require respawning.
-      if (defined $_params) {
-         for my $_p (qw(
+      if (defined (my $_p = $_params->{$_pid})) {
+         for my $_k (qw(
             RS interval stderr_file stdout_file user_error user_output
             job_delay submit_delay on_post_exit on_post_run user_args
             flush_file flush_stderr flush_stdout max_retries
          )) {
-            $_MCE->{$_p} = $_params->{$_p} if (exists $_params->{$_p});
+            $_MCE->{$_pid}->{$_k} = $_p->{$_k} if (exists $_p->{$_k});
          }
       }
    }
@@ -471,38 +489,35 @@ sub run (@) {
 
    if (defined $_input_data) {
       @_ = ();
-      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
-      delete $_MCE->{input_data};
+      $_MCE->{$_pid}->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{$_pid}->{input_data};
    }
    elsif (scalar @_) {
-      $_MCE->process({ chunk_size => $_chunk_size }, \@_);
-      delete $_MCE->{input_data};
+      $_MCE->{$_pid}->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{$_pid}->{input_data};
    }
    else {
-      if (defined $_params && exists $_params->{sequence}) {
-         $_MCE->run({
-            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+      if (defined $_params->{$_pid} && exists $_params->{$_pid}{sequence}) {
+         $_MCE->{$_pid}->run({
+             chunk_size => $_chunk_size,
+             sequence   => $_params->{$_pid}{sequence}
          }, 0);
-         if (exists $_params->{sequence_run}) {
-            delete $_params->{sequence_run};
-            delete $_params->{sequence};
+         if (exists $_params->{$_pid}{sequence_run}) {
+             delete $_params->{$_pid}{sequence_run};
+             delete $_params->{$_pid}{sequence};
          }
-         delete $_MCE->{sequence};
+         delete $_MCE->{$_pid}{sequence};
       }
    }
 
+   if ($^S || $ENV{'PERL_IPERL_RUNNING'} || $INC{'MCE/Hobo.pm'}) {
+      $_MCE->{$_pid}->shutdown(); # shutdown if in eval state
+      $_->DESTROY() for (@{ $_queue->{$_pid} });
+      delete $_queue->{$_pid};
+   }
+
    MCE::_restore_state();
 
-   if (exists $_MCE->{_rla_return}) {
-      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
-   }
-
-   if ($^S) {
-      ## shutdown if in eval state
-      MCE::_save_state(); $_MCE->shutdown(); MCE::_restore_state();
-      $_->DESTROY() for (@_queue); @_queue = ();
-   }
-
    return map { @{ $_ } } delete @_tmp{ 1 .. $_order_id - 1 }
       unless (defined $_aref);
 
@@ -524,13 +539,13 @@ sub _croak {
 
 sub _gen_user_tasks {
 
-   my ($_queue_ref, $_code_ref, $_mode_ref, $_name_ref, $_wrks_ref) = @_;
+   my ($_pid, $_queue_ref, $_code_ref, $_mode_ref, $_name_ref, $_wrks_ref) = @_;
 
-   @_user_tasks = ();
+   @{ $_user_tasks->{$_pid} } = ();
 
    ## For the code block farthest to the right.
 
-   push @_user_tasks, {
+   push @{ $_user_tasks->{$_pid} }, {
       task_name => $_name_ref->[-1],
       max_workers => $_wrks_ref->[-1],
 
@@ -564,7 +579,7 @@ sub _gen_user_tasks {
    for (my $_i = @{ $_code_ref } - 2; $_i > 0; $_i--) {
       my $_pos = $_i;
 
-      push @_user_tasks, {
+      push @{ $_user_tasks->{$_pid} }, {
          task_name => $_name_ref->[$_pos],
          max_workers => $_wrks_ref->[$_pos],
          gather => $_queue_ref->[$_pos - 1],
@@ -595,7 +610,7 @@ sub _gen_user_tasks {
 
    if (@{ $_code_ref } > 1) {
 
-      push @_user_tasks, {
+      push @{ $_user_tasks->{$_pid} }, {
          task_name => $_name_ref->[0],
          max_workers => $_wrks_ref->[0],
          gather => \&_preserve_order,
@@ -656,7 +671,7 @@ MCE::Stream - Parallel stream model for chaining multiple maps and greps
 
 =head1 VERSION
 
-This document describes MCE::Stream version 1.699_013
+This document describes MCE::Stream version 1.799_01
 
 =head1 SYNOPSIS
 
@@ -753,19 +768,20 @@ The following list options which may be overridden when loading the module.
    use JSON::XS qw( encode_json decode_json );
 
    use MCE::Stream
-         default_mode => 'grep',          ## Default 'map'
-         fast => 1,                       ## Default 0 (fast dequeue)
-         max_workers => 8,                ## Default 'auto'
-         chunk_size => 500,               ## Default 'auto'
-         tmp_dir => "/path/to/app/tmp",   ## $MCE::Signal::tmp_dir
-         freeze => \&encode_sereal,       ## \&Storable::freeze
-         thaw => \&decode_sereal          ## \&Storable::thaw
+       max_workers => 8,                # Default 'auto'
+       chunk_size => 500,               # Default 'auto'
+       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+       freeze => \&encode_sereal,       # \&Storable::freeze
+       thaw => \&decode_sereal,         # \&Storable::thaw
+       default_mode => 'grep',          # Default 'map'
+       fast => 1                        # Default 0 (fast dequeue)
    ;
 
 There is a simpler way to enable Sereal. The following will attempt to use
 Sereal if available, otherwise defaults to Storable for serialization.
+From MCE 1.800 onwards, this is done automatically.
 
-   use MCE::Stream Sereal => 1;
+   use MCE::Flow Sereal => 1;
 
 =head1 CUSTOMIZING MCE
 
@@ -978,7 +994,7 @@ longer needed.
 
 =head1 INDEX
 
-L<MCE|MCE>, L<MCE::Core>, L<MCE::Shared>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
