@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.699_013';
+our $VERSION = '1.799_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
@@ -187,38 +187,34 @@ sub _destroy_socks {
 sub _pipe_pair {
 
    my ($_obj, $_r_sock, $_w_sock, $_i) = @_;
-   my $_hndl; local ($|, $!);
-
-   # Doing $_obj->{$_r_sock}->autoflush(1) adds ~ 5% additional memory
-   # consumption from having to load IO::Handle.
+   local ($|, $!);
 
    if (defined $_i) {
       pipe($_obj->{$_r_sock}->[$_i], $_obj->{$_w_sock}->[$_i])
          or die "pipe: $!\n";
 
-      $_hndl = select $_obj->{$_r_sock}->[$_i]; $| = 1; # autoflush
-               select $_obj->{$_w_sock}->[$_i]; $| = 1;
+      # IO::Handle->autoflush does not exists on older Perl distros.
+      select((select($_obj->{$_r_sock}[$_i]), $| = 1)[0]);
+      select((select($_obj->{$_w_sock}[$_i]), $| = 1)[0]);
    }
    else {
       pipe($_obj->{$_r_sock}, $_obj->{$_w_sock})
          or die "pipe: $!\n";
 
-      $_hndl = select $_obj->{$_r_sock}; $| = 1; # ditto
-               select $_obj->{$_w_sock}; $| = 1;
+      # Ditto.
+      select((select($_obj->{$_r_sock}), $| = 1)[0]);
+      select((select($_obj->{$_w_sock}), $| = 1)[0]);
    }
 
-   select $_hndl;
-
    return;
 }
 
 sub _sock_pair {
 
    my ($_obj, $_r_sock, $_w_sock, $_i, $_size) = @_;
-   my $_hndl; local ($|, $!);
+   local ($|, $!);
 
    $_size = 16384 unless defined $_size;
-   # Ditto on not calling autoflush(1).
 
    if (defined $_i) {
       socketpair( $_obj->{$_r_sock}->[$_i], $_obj->{$_w_sock}->[$_i],
@@ -231,8 +227,9 @@ sub _sock_pair {
          setsockopt($_obj->{$_w_sock}->[$_i], SOL_SOCKET, SO_RCVBUF, int $_size);
       }
 
-      $_hndl = select $_obj->{$_r_sock}->[$_i]; $| = 1; # autoflush
-               select $_obj->{$_w_sock}->[$_i]; $| = 1;
+      # IO::Handle->autoflush does not exists on older Perl distros.
+      select((select($_obj->{$_r_sock}[$_i]), $| = 1)[0]);
+      select((select($_obj->{$_w_sock}[$_i]), $| = 1)[0]);
    }
    else {
       socketpair( $_obj->{$_r_sock}, $_obj->{$_w_sock},
@@ -245,41 +242,38 @@ sub _sock_pair {
          setsockopt($_obj->{$_w_sock}, SOL_SOCKET, SO_RCVBUF, int $_size);
       }
 
-      $_hndl = select $_obj->{$_r_sock}; $| = 1; # ditto
-               select $_obj->{$_w_sock}; $| = 1;
+      # Ditto.
+      select((select($_obj->{$_r_sock}), $| = 1)[0]);
+      select((select($_obj->{$_w_sock}), $| = 1)[0]);
    }
 
-   select $_hndl;
-
    return;
 }
 
 sub _sock_ready {
 
-   return if (
-      !defined $_[1] && !$INC{'MCE/Hobo.pm'} && defined $MCE::VERSION
-   );
+   my ($_bytes, $_socket, $_timeout ) = ("\x00\x00\x00\x00", @_);
+   my ($_cnt, $_retries, $_ptr_bytes) = (1, 0, unpack('I', pack('P', $_bytes)));
 
-   my ($_val_bytes, $_socket, $_timeout) = ("\x00\x00\x00\x00", @_);
-   my $_ptr_bytes = unpack('I', pack('P', $_val_bytes));
+   $_timeout += time if $_timeout;
 
-   if ($_timeout) {
-      $_timeout += time();
-      while (1) {
-         ioctl($_socket, 0x4004667f, $_ptr_bytes);  # MSWin32 FIONREAD
-       # return '' if unpack('I', $_val_bytes);     # unpack isn't needed here
-         return '' if $_val_bytes ne $_zero_bytes;  # this completes 2x faster
-         return 1  if time() > $_timeout;
-         sleep 0.045;
-      }
-   }
-   else {
-      my $_count = 0;
-      while (1) {
-         ioctl($_socket, 0x4004667f, $_ptr_bytes);  # Ditto
-         return if $_val_bytes ne $_zero_bytes;
-         $_count = 0, sleep 0.008 if ++$_count > 1618;
+   while (1) {
+      ioctl($_socket, 0x4004667f, $_ptr_bytes);   # MSWin32 FIONREAD
+
+    # return '' if unpack('I', $_bytes);          # unpack not necessary
+      return '' if $_bytes ne $_zero_bytes;       # string compare, 2x faster
+      return 1  if $_timeout && time > $_timeout;
+
+      # delay to not consume a CPU from non-blocking ioctl
+      if ($_cnt) {
+         if (++$_cnt > 900) {
+            $_cnt = 1, sleep 0.015;
+            $_cnt = 0 if ++$_retries == 2;
+         }
+         next;
       }
+
+      sleep 0.030;
    }
 }
 
@@ -334,18 +328,18 @@ sub _parse_chunk_size {
       if ( (defined $_params && ref $_params->{input_data} eq 'CODE') ||
            (defined $_input_data && ref $_input_data eq 'CODE')
       ) {
-         ## Iterators may optionally use chunk_size to determine how much
-         ## to return per iteration. The default is 1 for MCE Models, same
-         ## as for the Core API. The user_func receives an array_ref
-         ## regardless if 1 or higher.
-         ##
-         ## sub make_iter {
-         ##    ...
-         ##    return sub {
-         ##       my ($chunk_size) = @_;
-         ##       ...
-         ##    };
-         ## }
+         # Iterators may optionally use chunk_size to determine how much
+         # to return per iteration. The default is 1 for MCE Models, same
+         # as for the Core API. The user_func receives an array_ref
+         # regardless if 1 or higher.
+         #
+         # sub make_iter {
+         #    ...
+         #    return sub {
+         #       my ($chunk_size) = @_;
+         #       ...
+         #    };
+         # }
          return 1;
       }
 
@@ -380,14 +374,14 @@ sub _parse_chunk_size {
          } elsif ($_ref eq '') {
             $_size = -s $_params->{_file};
          } else {
-            $_size = 0; $_chunk_size = 245_760;
+            $_size = 0; $_chunk_size = 393_216;  # 384K
          }
 
          $_is_file = 1;
       }
       elsif (defined $_input_data) {
          if (ref $_input_data eq 'GLOB' || ref($_input_data) =~ /^IO::/) {
-            $_is_file = 1; $_size = 0; $_chunk_size = 245_760;
+            $_is_file = 1; $_size = 0; $_chunk_size = 393_216;  # 384K
          }
          elsif (ref $_input_data eq 'SCALAR') {
             $_is_file = 1; $_size = length ${ $_input_data };
@@ -397,7 +391,7 @@ sub _parse_chunk_size {
       if (defined $_is_file) {
          if ($_size) {
             $_chunk_size = int($_size / $_max_workers / 24 + 0.5);
-            $_chunk_size = 4_194_304 if $_chunk_size > 4_194_304;  ## 4M
+            $_chunk_size = 5_242_880 if $_chunk_size > 5_242_880;  # 5M
             $_chunk_size = 2 if $_chunk_size <= 8192;
          }
       }
@@ -427,7 +421,7 @@ MCE::Util - Utility functions
 
 =head1 VERSION
 
-This document describes MCE::Util version 1.699_013
+This document describes MCE::Util version 1.799_01
 
 =head1 SYNOPSIS
 
@@ -478,14 +472,14 @@ In summary:
  4. One can specify max_workers explicity to a hard value
  5. MCE::Util::get_ncpu returns the actual # of lcores
 
-=head1 ACKNOWLEDGEMENTS
+=head1 ACKNOWLEDGMENTS
 
 The portable code for detecting the number of processors was adopted from
 L<Test::Smoke::SysInfo>.
 
 =head1 INDEX
 
-L<MCE|MCE>, L<MCE::Core>, L<MCE::Shared>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
